@@ -119,6 +119,7 @@ class AstInterpreter(interpreterbase.InterpreterBase):
                            'find_library': self.func_do_nothing,
                            'subdir_done': self.func_do_nothing,
                            'alias_target': self.func_do_nothing,
+                           'summary': self.func_do_nothing,
                            })
 
     def func_do_nothing(self, node, args, kwargs):
@@ -289,7 +290,7 @@ class AstInterpreter(interpreterbase.InterpreterBase):
             l = quick_resolve(node.left)
             r = quick_resolve(node.right)
             if isinstance(l, str) and isinstance(r, str):
-                result = l + r # String concatination detected
+                result = l + r # String concatenation detected
             else:
                 result = self.flatten_args(l, include_unknown_args, id_loop_detect) + self.flatten_args(r, include_unknown_args, id_loop_detect)
 
