@@ -26,7 +26,6 @@ from ... import mesonlib
 from ... import mlog
 
 if typing.TYPE_CHECKING:
-    from ..compilers import CompilerType
     from ...coredata import UserOption  # noqa: F401
     from ...environment import Environment
 
@@ -101,14 +100,15 @@ def gnulike_default_include_dirs(compiler: typing.Tuple[str], lang: str) -> typi
     p = subprocess.Popen(
         cmd,
         stdin=subprocess.DEVNULL,
-        stderr=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
         stdout=subprocess.PIPE,
         env=env
     )
-    stderr = p.stderr.read().decode('utf-8', errors='replace')
+    stdout = p.stdout.read().decode('utf-8', errors='replace')
     parse_state = 0
     paths = []
-    for line in stderr.split('\n'):
+    for line in stdout.split('\n'):
+        line = line.strip(' \n\r\t')
         if parse_state == 0:
             if line == '#include "..." search starts here:':
                 parse_state = 1
@@ -116,14 +116,16 @@ def gnulike_default_include_dirs(compiler: typing.Tuple[str], lang: str) -> typi
             if line == '#include <...> search starts here:':
                 parse_state = 2
             else:
-                paths.append(line[1:])
+                paths.append(line)
         elif parse_state == 2:
             if line == 'End of search list.':
                 break
             else:
-                paths.append(line[1:])
+                paths.append(line)
     if not paths:
         mlog.warning('No include directory found parsing "{cmd}" output'.format(cmd=" ".join(cmd)))
+    # Append a normalized copy of paths to make path lookup easier
+    paths += [os.path.normpath(x) for x in paths]
     return paths
 
 
@@ -137,19 +139,18 @@ class GnuLikeCompiler(metaclass=abc.ABCMeta):
 
     LINKER_PREFIX = '-Wl,'
 
-    def __init__(self, compiler_type: 'CompilerType'):
-        self.compiler_type = compiler_type
+    def __init__(self):
         self.base_options = ['b_pch', 'b_lto', 'b_pgo', 'b_sanitize', 'b_coverage',
                              'b_ndebug', 'b_staticpic', 'b_pie']
-        if not (self.compiler_type.is_windows_compiler or mesonlib.is_openbsd()):
+        if not (self.info.is_windows() or self.info.is_cygwin() or self.info.is_openbsd()):
             self.base_options.append('b_lundef')
-        if not self.compiler_type.is_windows_compiler:
+        if not self.info.is_windows() or self.info.is_cygwin():
             self.base_options.append('b_asneeded')
         # All GCC-like backends can do assembly
         self.can_compile_suffixes.add('s')
 
     def get_pic_args(self) -> typing.List[str]:
-        if self.compiler_type.is_osx_compiler or self.compiler_type.is_windows_compiler:
+        if self.info.is_windows() or self.info.is_cygwin() or self.info.is_darwin():
             return [] # On Window and OS X, pic is always on.
         return ['-fPIC']
 
@@ -191,7 +192,7 @@ class GnuLikeCompiler(metaclass=abc.ABCMeta):
             raise RuntimeError('Module definitions file should be str')
         # On Windows targets, .def files may be specified on the linker command
         # line like an object file.
-        if self.compiler_type.is_windows_compiler:
+        if self.info.is_windows() or self.info.is_cygwin():
             return [defsfile]
         # For other targets, discard the .def file.
         return []
@@ -206,7 +207,7 @@ class GnuLikeCompiler(metaclass=abc.ABCMeta):
         return ['-fprofile-use', '-fprofile-correction']
 
     def get_gui_app_args(self, value: bool) -> typing.List[str]:
-        if self.compiler_type.is_windows_compiler:
+        if self.info.is_windows() or self.info.is_cygwin():
             return ['-mwindows' if value else '-mconsole']
         return []
 
@@ -301,6 +302,10 @@ class GnuLikeCompiler(metaclass=abc.ABCMeta):
             return ['-isystem' + path]
         return ['-I' + path]
 
+    @classmethod
+    def use_linker_args(cls, linker: str) -> typing.List[str]:
+        return ['-fuse-ld={}'.format(linker)]
+
 
 class GnuCompiler(GnuLikeCompiler):
     """
@@ -308,8 +313,8 @@ class GnuCompiler(GnuLikeCompiler):
     Compilers imitating GCC (Clang/Intel) should use the GnuLikeCompiler ABC.
     """
 
-    def __init__(self, compiler_type: 'CompilerType', defines: typing.Dict[str, str]):
-        super().__init__(compiler_type)
+    def __init__(self, defines: typing.Dict[str, str]):
+        super().__init__()
         self.id = 'gcc'
         self.defines = defines or {}
         self.base_options.append('b_colorout')
@@ -343,3 +348,15 @@ class GnuCompiler(GnuLikeCompiler):
 
     def openmp_flags(self) -> typing.List[str]:
         return ['-fopenmp']
+
+    def has_arguments(self, args, env, code, mode):
+        # For some compiler command line arguments, the GNU compilers will
+        # emit a warning on stderr indicating that an option is valid for a
+        # another language, but still complete with exit_success
+        with self._build_wrapper(code, env, args, None, mode, disable_cache=False, want_output=True) as p:
+            result = p.returncode == 0
+            if self.language in {'cpp', 'objcpp'} and 'is valid for C/ObjC' in p.stde:
+                result = False
+            if self.language in {'c', 'objc'} and 'is valid for C++/ObjC++' in p.stde:
+                result = False
+        return result, p.cached
