@@ -83,7 +83,7 @@ class VisualStudioLikeCompiler(metaclass=abc.ABCMeta):
 
     A number of compilers attempt to mimic MSVC, with varying levels of
     success, such as Clang-CL and ICL (the Intel C/C++ Compiler for Windows).
-    This classs implements as much common logic as possible.
+    This class implements as much common logic as possible.
     """
 
     std_warn_args = ['/W3']
@@ -111,6 +111,8 @@ class VisualStudioLikeCompiler(metaclass=abc.ABCMeta):
         '3': ['/W4'],
     }  # type: typing.Dict[str, typing.List[str]]
 
+    INVOKES_LINKER = False
+
     def __init__(self, target: str):
         self.base_options = ['b_pch', 'b_ndebug', 'b_vscrt'] # FIXME add lto, pgo and the like
         self.target = target
@@ -217,7 +219,9 @@ class VisualStudioLikeCompiler(metaclass=abc.ABCMeta):
             # -pthread is only valid for GCC
             if i in ('-mms-bitfields', '-pthread'):
                 continue
-            if i.startswith('-L'):
+            if i.startswith('-LIBPATH:'):
+                i = '/LIBPATH:' + i[9:]
+            elif i.startswith('-L'):
                 i = '/LIBPATH:' + i[2:]
             # Translate GNU-style -lfoo library name to the import library
             elif i.startswith('-l'):
@@ -250,7 +254,7 @@ class VisualStudioLikeCompiler(metaclass=abc.ABCMeta):
     def native_args_to_unix(cls, args: typing.List[str]) -> typing.List[str]:
         result = []
         for arg in args:
-            if arg.startswith('/LIBPATH:'):
+            if arg.startswith(('/LIBPATH:', '-LIBPATH:')):
                 result.append('-L' + arg[9:])
             elif arg.endswith(('.a', '.lib')) and not os.path.isabs(arg):
                 result.append('-l' + arg)
@@ -379,3 +383,7 @@ class VisualStudioLikeCompiler(metaclass=abc.ABCMeta):
 
     def get_argument_syntax(self) -> str:
         return 'msvc'
+
+    @classmethod
+    def use_linker_args(cls, linker: str) -> typing.List[str]:
+        return []
