@@ -27,18 +27,20 @@ import ast
 import argparse
 import configparser
 from typing import (
-    Any, Dict, Generic, Iterable, List, Optional, Type, TypeVar, Union
+    Any, Dict, Generic, Iterable, Iterator, List, MutableMapping, Optional, Tuple, Type, TypeVar, Union,
+    TYPE_CHECKING,
 )
-import typing
 import enum
 import shlex
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from . import dependencies
+    from .compilers import Compiler  # noqa: F401
+    from .environment import Environment
 
-    OptionDictType = typing.Dict[str, 'UserOption[Any]']
+    OptionDictType = Dict[str, 'UserOption[Any]']
 
-version = '0.52.1'
+version = '0.53.0'
 backendlist = ['ninja', 'vs', 'vs2010', 'vs2015', 'vs2017', 'vs2019', 'xcode']
 
 default_yielding = False
@@ -124,7 +126,7 @@ class UserIntegerOption(UserOption[int]):
         try:
             return int(valuestring)
         except ValueError:
-            raise MesonException('Value string "%s" is not convertable to an integer.' % valuestring)
+            raise MesonException('Value string "%s" is not convertible to an integer.' % valuestring)
 
 class UserUmaskOption(UserIntegerOption, UserOption[Union[str, int]]):
     def __init__(self, description, value, yielding=None):
@@ -170,7 +172,7 @@ class UserArrayOption(UserOption[List[str]]):
         self.allow_dups = allow_dups
         self.value = self.validate_value(value, user_input=user_input)
 
-    def validate_value(self, value, user_input=True) -> List[str]:
+    def validate_value(self, value, user_input: bool = True) -> List[str]:
         # User input is for options defined on the command line (via -D
         # options). Users can put their input in as a comma separated
         # string, but for defining options in meson_options.txt the format
@@ -180,7 +182,10 @@ class UserArrayOption(UserOption[List[str]]):
 
         if isinstance(value, str):
             if value.startswith('['):
-                newvalue = ast.literal_eval(value)
+                try:
+                    newvalue = ast.literal_eval(value)
+                except ValueError:
+                    raise MesonException('malformed option {}'.format(value))
             elif value == '':
                 newvalue = []
             else:
@@ -191,7 +196,7 @@ class UserArrayOption(UserOption[List[str]]):
         elif isinstance(value, list):
             newvalue = value
         else:
-            raise MesonException('"{0}" should be a string array, but it is not'.format(str(newvalue)))
+            raise MesonException('"{}" should be a string array, but it is not'.format(newvalue))
 
         if not self.allow_dups and len(set(newvalue)) != len(newvalue):
             msg = 'Duplicated values in array option is deprecated. ' \
@@ -231,9 +236,9 @@ def load_configs(filenames: List[str]) -> configparser.ConfigParser:
     return config
 
 
-if typing.TYPE_CHECKING:
-    CacheKeyType = typing.Tuple[typing.Tuple[typing.Any, ...], ...]
-    SubCacheKeyType = typing.Tuple[typing.Any, ...]
+if TYPE_CHECKING:
+    CacheKeyType = Tuple[Tuple[Any, ...], ...]
+    SubCacheKeyType = Tuple[Any, ...]
 
 
 class DependencyCacheType(enum.Enum):
@@ -257,7 +262,7 @@ class DependencySubCache:
 
     def __init__(self, type_: DependencyCacheType):
         self.types = [type_]
-        self.__cache = {}  # type: typing.Dict[SubCacheKeyType, dependencies.Dependency]
+        self.__cache = {}  # type: Dict[SubCacheKeyType, dependencies.Dependency]
 
     def __getitem__(self, key: 'SubCacheKeyType') -> 'dependencies.Dependency':
         return self.__cache[key]
@@ -268,7 +273,7 @@ class DependencySubCache:
     def __contains__(self, key: 'SubCacheKeyType') -> bool:
         return key in self.__cache
 
-    def values(self) -> typing.Iterable['dependencies.Dependency']:
+    def values(self) -> Iterable['dependencies.Dependency']:
         return self.__cache.values()
 
 
@@ -280,12 +285,12 @@ class DependencyCache:
     successfully lookup by providing a simple get/put interface.
     """
 
-    def __init__(self, builtins_per_machine: PerMachine[typing.Dict[str, UserOption[typing.Any]]], for_machine: MachineChoice):
-        self.__cache = OrderedDict()  # type: typing.MutableMapping[CacheKeyType, DependencySubCache]
+    def __init__(self, builtins_per_machine: PerMachine[Dict[str, UserOption[Any]]], for_machine: MachineChoice):
+        self.__cache = OrderedDict()  # type: MutableMapping[CacheKeyType, DependencySubCache]
         self.__builtins_per_machine = builtins_per_machine
         self.__for_machine = for_machine
 
-    def __calculate_subkey(self, type_: DependencyCacheType) -> typing.Tuple[typing.Any, ...]:
+    def __calculate_subkey(self, type_: DependencyCacheType) -> Tuple[Any, ...]:
         if type_ is DependencyCacheType.PKG_CONFIG:
             return tuple(self.__builtins_per_machine[self.__for_machine]['pkg_config_path'].value)
         elif type_ is DependencyCacheType.CMAKE:
@@ -293,7 +298,7 @@ class DependencyCache:
         assert type_ is DependencyCacheType.OTHER, 'Someone forgot to update subkey calculations for a new type'
         return tuple()
 
-    def __iter__(self) -> typing.Iterator['CacheKeyType']:
+    def __iter__(self) -> Iterator['CacheKeyType']:
         return self.keys()
 
     def put(self, key: 'CacheKeyType', dep: 'dependencies.Dependency') -> None:
@@ -303,7 +308,7 @@ class DependencyCache:
         subkey = self.__calculate_subkey(t)
         self.__cache[key][subkey] = dep
 
-    def get(self, key: 'CacheKeyType') -> typing.Optional['dependencies.Dependency']:
+    def get(self, key: 'CacheKeyType') -> Optional['dependencies.Dependency']:
         """Get a value from the cache.
 
         If there is no cache entry then None will be returned.
@@ -321,14 +326,14 @@ class DependencyCache:
                 pass
         return None
 
-    def values(self) -> typing.Iterator['dependencies.Dependency']:
+    def values(self) -> Iterator['dependencies.Dependency']:
         for c in self.__cache.values():
             yield from c.values()
 
-    def keys(self) -> typing.Iterator['CacheKeyType']:
+    def keys(self) -> Iterator['CacheKeyType']:
         return iter(self.__cache.keys())
 
-    def items(self) -> typing.Iterator[typing.Tuple['CacheKeyType', typing.List['dependencies.Dependency']]]:
+    def items(self) -> Iterator[Tuple['CacheKeyType', List['dependencies.Dependency']]]:
         for k, v in self.__cache.items():
             vs = []
             for t in v.types:
@@ -390,9 +395,9 @@ class CoreData:
         if not filenames:
             return []
 
-        found_invalid = []  # type: typing.List[str]
-        missing = []        # type: typing.List[str]
-        real = []           # type: typing.List[str]
+        found_invalid = []  # type: List[str]
+        missing = []        # type: List[str]
+        real = []           # type: List[str]
         for i, f in enumerate(filenames):
             f = os.path.expanduser(os.path.expandvars(f))
             if os.path.exists(f):
@@ -412,7 +417,7 @@ class CoreData:
                     real.append(copy)
 
                     # Also replace the command line argument, as the pipe
-                    # probably wont exist on reconfigure
+                    # probably won't exist on reconfigure
                     filenames[i] = copy
                     continue
             if sys.platform != 'win32':
@@ -618,8 +623,7 @@ class CoreData:
                 except MesonException as e:
                     raise type(e)(('Validation failed for option %s: ' % option_name) + str(e)) \
                         .with_traceback(sys.exc_info()[2])
-        else:
-            raise MesonException('Tried to validate unknown option %s.' % option_name)
+        raise MesonException('Tried to validate unknown option %s.' % option_name)
 
     def get_external_args(self, for_machine: MachineChoice, lang):
         return self.compiler_options[for_machine][lang + '_args'].value
@@ -659,7 +663,6 @@ class CoreData:
         if not self.is_cross_build():
             options = self.strip_build_option_names(options)
         # Set prefix first because it's needed to sanitize other options
-        prefix = self.builtins['prefix'].value
         if 'prefix' in options:
             prefix = self.sanitize_prefix(options['prefix'])
             self.builtins['prefix'].set_value(prefix)
@@ -741,13 +744,29 @@ class CoreData:
 
         self.set_options(options, subproject=subproject)
 
-    def process_new_compiler(self, lang: str, comp, env):
+    def add_lang_args(self, lang: str, comp: Type['Compiler'],
+                      for_machine: MachineChoice, env: 'Environment') -> None:
+        """Add global language arguments that are needed before compiler/linker detection."""
+        from .compilers import compilers
+
+        optprefix = lang + '_'
+        for k, o in compilers.get_global_options(lang, comp, env.properties[for_machine]).items():
+            if not k.startswith(optprefix):
+                raise MesonException('Internal error, %s has incorrect prefix.' % k)
+            # prefixed compiler options affect just this machine
+            opt_prefix = for_machine.get_prefix()
+            if opt_prefix + k in env.cmd_line_options:
+                o.set_value(env.cmd_line_options[opt_prefix + k])
+            self.compiler_options[for_machine].setdefault(k, o)
+
+    def process_new_compiler(self, lang: str, comp: Type['Compiler'], env: 'Environment') -> None:
         from . import compilers
 
         self.compilers[comp.for_machine][lang] = comp
+        enabled_opts = []
 
         optprefix = lang + '_'
-        for k, o in comp.get_and_default_options(env.properties[comp.for_machine]).items():
+        for k, o in comp.get_options().items():
             if not k.startswith(optprefix):
                 raise MesonException('Internal error, %s has incorrect prefix.' % k)
             # prefixed compiler options affect just this machine
@@ -776,7 +795,7 @@ class CmdLineFileParser(configparser.ConfigParser):
     def __init__(self):
         # We don't want ':' as key delimiter, otherwise it would break when
         # storing subproject options like "subproject:option=value"
-        super().__init__(delimiters=['='])
+        super().__init__(delimiters=['='], interpolation=None)
 
 def get_cmd_line_file(build_dir):
     return os.path.join(build_dir, 'meson-private', 'cmd_line.txt')
