@@ -153,7 +153,7 @@ class ArLinker(StaticLinker):
         return [target]
 
 
-class ArmarLinker(ArLinker):
+class ArmarLinker(ArLinker):  # lgtm [py/missing-call-to-init]
 
     def __init__(self, exelist: typing.List[str]):
         StaticLinker.__init__(self, exelist)
@@ -161,7 +161,7 @@ class ArmarLinker(ArLinker):
         self.std_args = ['-csr']
 
     def can_linker_accept_rsp(self) -> bool:
-        # armar cann't accept arguments using the @rsp syntax
+        # armar can't accept arguments using the @rsp syntax
         return False
 
 
@@ -247,17 +247,21 @@ class DynamicLinker(metaclass=abc.ABCMeta):
     }  # type: typing.Dict[str, typing.List[str]]
 
     def _apply_prefix(self, arg: str) -> typing.List[str]:
-        if isinstance(self.prefix_arg, str):
+        if self.prefix_arg is None:
+            return [arg]
+        elif isinstance(self.prefix_arg, str):
             return [self.prefix_arg + arg]
         return self.prefix_arg + [arg]
 
     def __init__(self, exelist: typing.List[str], for_machine: mesonlib.MachineChoice,
-                 id_: str, prefix_arg: str, *, version: str = 'unknown version'):
+                 id_: str, prefix_arg: typing.Union[str, typing.List[str]],
+                 always_args: typing.List[str], *, version: str = 'unknown version'):
         self.exelist = exelist
         self.for_machine = for_machine
         self.version = version
         self.id = id_
         self.prefix_arg = prefix_arg
+        self.always_args = always_args
 
     def __repr__(self) -> str:
         return '<{}: v{} `{}`>'.format(type(self).__name__, self.version, ' '.join(self.exelist))
@@ -276,7 +280,7 @@ class DynamicLinker(metaclass=abc.ABCMeta):
         return mesonlib.is_windows()
 
     def get_always_args(self) -> typing.List[str]:
-        return []
+        return self.always_args.copy()
 
     def get_lib_prefix(self) -> str:
         return ''
@@ -340,10 +344,6 @@ class DynamicLinker(metaclass=abc.ABCMeta):
         raise mesonlib.EnvironmentException(
             'Linker {} does not support allow undefined'.format(self.id))
 
-    def invoked_by_compiler(self) -> bool:
-        """True if meson uses the compiler to invoke the linker."""
-        return True
-
     @abc.abstractmethod
     def get_output_args(self, outname: str) -> typing.List[str]:
         pass
@@ -395,6 +395,11 @@ class DynamicLinker(metaclass=abc.ABCMeta):
                          install_rpath: str) -> typing.List[str]:
         return []
 
+    def get_soname_args(self, env: 'Environment', prefix: str, shlib_name: str,
+                        suffix: str, soversion: str, darwin_versions: typing.Tuple[str, str],
+                        is_shared_module: bool) -> typing.List[str]:
+        return []
+
 
 class PosixDynamicLinkerMixin:
 
@@ -412,15 +417,15 @@ class PosixDynamicLinkerMixin:
         return ['-shared']
 
     def get_search_args(self, dirname: str) -> typing.List[str]:
-        return ['-L', dirname]
+        return ['-L' + dirname]
 
 
 class GnuLikeDynamicLinkerMixin:
 
     """Mixin class for dynamic linkers that provides gnu-like interface.
 
-    This acts as a base for the GNU linkers (bfd and gold), the Intel Xild
-    (which comes with ICC), LLVM's lld, and other linkers like GNU-ld.
+    This acts as a base for the GNU linkers (bfd and gold), LLVM's lld, and
+    other linkers like GNU-ld.
     """
 
     _BUILDTYPE_ARGS = {
@@ -459,10 +464,6 @@ class GnuLikeDynamicLinkerMixin:
             return []
         return ['-fsanitize=' + value]
 
-    def invoked_by_compiler(self) -> bool:
-        """True if meson uses the compiler to invoke the linker."""
-        return True
-
     def get_coverage_args(self) -> typing.List[str]:
         return ['--coverage']
 
@@ -573,6 +574,9 @@ class AppleDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
     def get_std_shared_module_args(self, options: 'OptionDictType') -> typing.List[str]:
         return ['-bundle'] + self._apply_prefix('-undefined,dynamic_lookup')
 
+    def get_pie_args(self) -> typing.List[str]:
+        return ['-pie']
+
     def get_link_whole_for(self, args: typing.List[str]) -> typing.List[str]:
         result = []  # type: typing.List[str]
         for a in args:
@@ -580,11 +584,19 @@ class AppleDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
             result.append(a)
         return result
 
+    def get_coverage_args(self) -> typing.List[str]:
+        return ['--coverage']
+
+    def sanitizer_args(self, value: str) -> typing.List[str]:
+        if value == 'none':
+            return []
+        return ['-fsanitize=' + value]
+
     def no_undefined_args(self) -> typing.List[str]:
         return self._apply_prefix('-undefined,error')
 
     def get_always_args(self) -> typing.List[str]:
-        return self._apply_prefix('-headerpad_max_install_names')
+        return self._apply_prefix('-headerpad_max_install_names') + super().get_always_args()
 
     def bitcode_args(self) -> typing.List[str]:
         return self._apply_prefix('-bitcode_bundle')
@@ -645,26 +657,6 @@ class LLVMDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, Dyna
     pass
 
 
-class XildLinuxDynamicLinker(GnuLikeDynamicLinkerMixin, PosixDynamicLinkerMixin, DynamicLinker):
-
-    """Representation of Intel's Xild linker.
-
-    This is only the linux-like linker which dispatches to Gnu ld.
-    """
-
-    pass
-
-
-class XildAppleDynamicLinker(AppleDynamicLinker):
-
-    """Representation of Intel's Xild linker.
-
-    This is the apple linker, which dispatches to Apple's ld.
-    """
-
-    pass
-
-
 class CcrxDynamicLinker(DynamicLinker):
 
     """Linker for Renesis CCrx compiler."""
@@ -704,7 +696,7 @@ class ArmDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
 
     def __init__(self, for_machine: mesonlib.MachineChoice,
                  *, version: str = 'unknown version'):
-        super().__init__(['armlink'], for_machine, 'armlink', '',
+        super().__init__(['armlink'], for_machine, 'armlink', '', [],
                          version=version)
 
     def get_accepts_rsp(self) -> bool:
@@ -787,11 +779,10 @@ class VisualStudioLikeLinkerMixin:
 
     def __init__(self, *args, direct: bool = True, machine: str = 'x86', **kwargs):
         super().__init__(*args, **kwargs)
-        self.direct = direct
         self.machine = machine
 
     def invoked_by_compiler(self) -> bool:
-        return self.direct
+        return not self.direct
 
     def get_debug_crt_args(self) -> typing.List[str]:
         """Arguments needed to select a debug crt for the linker.
@@ -807,7 +798,7 @@ class VisualStudioLikeLinkerMixin:
         return self._apply_prefix('/MACHINE:' + self.machine) + self._apply_prefix('/OUT:' + outputname)
 
     def get_always_args(self) -> typing.List[str]:
-        return self._apply_prefix('/nologo')
+        return self._apply_prefix('/nologo') + super().get_always_args()
 
     def get_search_args(self, dirname: str) -> typing.List[str]:
         return self._apply_prefix('/LIBPATH:' + dirname)
@@ -829,8 +820,7 @@ class VisualStudioLikeLinkerMixin:
         return l
 
     def get_allow_undefined_args(self) -> typing.List[str]:
-        # link.exe
-        return self._apply_prefix('/FORCE:UNRESOLVED')
+        return []
 
     def get_soname_args(self, env: 'Environment', prefix: str, shlib_name: str,
                         suffix: str, soversion: str, darwin_versions: typing.Tuple[str, str],
@@ -842,33 +832,35 @@ class MSVCDynamicLinker(VisualStudioLikeLinkerMixin, DynamicLinker):
 
     """Microsoft's Link.exe."""
 
-    def __init__(self, for_machine: mesonlib.MachineChoice, *,
+    def __init__(self, for_machine: mesonlib.MachineChoice, always_args: typing.List[str], *,
                  exelist: typing.Optional[typing.List[str]] = None,
                  prefix: typing.Union[str, typing.List[str]] = '',
-                 machine: str = 'x86', version: str = 'unknown version'):
+                 machine: str = 'x86', version: str = 'unknown version',
+                 direct: bool = True):
         super().__init__(exelist or ['link.exe'], for_machine, 'link',
-                         prefix, machine=machine, version=version)
+                         prefix, always_args, machine=machine, version=version, direct=direct)
 
 
 class ClangClDynamicLinker(VisualStudioLikeLinkerMixin, DynamicLinker):
 
     """Clang's lld-link.exe."""
 
-    def __init__(self, for_machine: mesonlib.MachineChoice, *,
+    def __init__(self, for_machine: mesonlib.MachineChoice, always_args: typing.List[str], *,
                  exelist: typing.Optional[typing.List[str]] = None,
                  prefix: typing.Union[str, typing.List[str]] = '',
-                 version: str = 'unknown version'):
-        super().__init__(exelist or ['lld-link.exe'], for_machine,
-                         'lld-link', prefix, version=version)
+                 machine: str = 'x86', version: str = 'unknown version',
+                 direct: bool = True):
+        super().__init__(exelist or ['lld-link.exe'], for_machine, 'lld-link',
+                         prefix, always_args, machine=machine, version=version, direct=direct)
 
 
 class XilinkDynamicLinker(VisualStudioLikeLinkerMixin, DynamicLinker):
 
     """Intel's Xilink.exe."""
 
-    def __init__(self, for_machine: mesonlib.MachineChoice,
+    def __init__(self, for_machine: mesonlib.MachineChoice, always_args: typing.List[str],
                  *, version: str = 'unknown version'):
-        super().__init__(['xilink.exe'], for_machine, 'xilink', '', version=version)
+        super().__init__(['xilink.exe'], for_machine, 'xilink', '', always_args, version=version)
 
 
 class SolarisDynamicLinker(PosixDynamicLinkerMixin, DynamicLinker):
@@ -925,12 +917,12 @@ class OptlinkDynamicLinker(VisualStudioLikeLinkerMixin, DynamicLinker):
                  *, version: str = 'unknown version'):
         # Use optlink instead of link so we don't interfer with other link.exe
         # implementations.
-        super().__init__(['optlink.exe'], for_machine, 'optlink', prefix_arg='', version=version)
+        super().__init__(['optlink.exe'], for_machine, 'optlink', '', [], version=version)
 
     def get_allow_undefined_args(self) -> typing.List[str]:
         return []
 
-class CudaLinker(DynamicLinker):
+class CudaLinker(PosixDynamicLinkerMixin, DynamicLinker):
     """Cuda linker (nvlink)"""
     @staticmethod
     def parse_version():
@@ -964,12 +956,6 @@ class CudaLinker(DynamicLinker):
         from .compilers import CudaCompiler
         return CudaCompiler.LINKER_PREFIX
 
-    def get_output_args(self, outname: str) -> typing.List[str]:
-        return ['-o', outname]
-
-    def get_search_args(self, dirname: str) -> typing.List[str]:
-        return ['-L', dirname]
-
     def fatal_warnings(self) -> typing.List[str]:
         return ['--warning-as-error']
 
@@ -980,6 +966,3 @@ class CudaLinker(DynamicLinker):
                         suffix: str, soversion: str, darwin_versions: typing.Tuple[str, str],
                         is_shared_module: bool) -> typing.List[str]:
         return []
-
-    def get_std_shared_lib_args(self) -> typing.List[str]:
-        return ['-shared']
