@@ -21,6 +21,7 @@ import platform, subprocess, operator, os, shlex, shutil, re
 import collections
 from enum import Enum
 from functools import lru_cache, update_wrapper
+from itertools import tee, filterfalse
 import typing
 import uuid
 
@@ -397,7 +398,7 @@ class PerMachineDefaultable(PerMachine[typing.Optional[_T]]):
         super().__init__(None, None)
 
     def default_missing(self) -> "PerMachine[typing.Optional[_T]]":
-        """Default host to buid
+        """Default host to build
 
         This allows just specifying nothing in the native case, and just host in the
         cross non-compiler case.
@@ -415,7 +416,7 @@ class PerThreeMachineDefaultable(PerMachineDefaultable, PerThreeMachine[typing.O
         PerThreeMachine.__init__(self, None, None, None)
 
     def default_missing(self) -> "PerThreeMachine[typing.Optional[_T]]":
-        """Default host to buid and target to host.
+        """Default host to build and target to host.
 
         This allows just specifying nothing in the native case, just host in the
         cross non-compiler case, and just target in the native-built
@@ -687,9 +688,9 @@ def default_prefix():
 
 def get_library_dirs() -> typing.List[str]:
     if is_windows():
-        return ['C:/mingw/lib'] # TODO: get programatically
+        return ['C:/mingw/lib'] # TODO: get programmatically
     if is_osx():
-        return ['/usr/lib'] # TODO: get programatically
+        return ['/usr/lib'] # TODO: get programmatically
     # The following is probably Debian/Ubuntu specific.
     # /usr/local/lib is first because it contains stuff
     # installed by the sysadmin and is probably more up-to-date
@@ -769,7 +770,7 @@ if is_windows():
         result += (num_backslashes * 2) * '\\' + '"'
         return result
 
-    def split_args(cmd):
+    def split_args(cmd: typing.Sequence[str]) -> typing.List[str]:
         result = []
         arg = ''
         num_backslashes = 0
@@ -975,7 +976,9 @@ def replace_if_different(dst, dst_tmp):
     else:
         os.unlink(dst_tmp)
 
-def listify(item, flatten=True, unholder=False):
+def listify(item: typing.Any,
+            flatten: bool = True,
+            unholder: bool = False) -> typing.List[typing.Any]:
     '''
     Returns a list with all args embedded in a list if they are not a list.
     This function preserves order.
@@ -1051,6 +1054,12 @@ def expand_arguments(args):
             return None
     return expended_args
 
+def partition(pred, iterable):
+    'Use a predicate to partition entries into false entries and true entries'
+    # partition(is_odd, range(10)) --> 0 2 4 6 8   and  1 3 5 7 9
+    t1, t2 = tee(iterable)
+    return filterfalse(pred, t1), filter(pred, t2)
+
 def Popen_safe(args: typing.List[str], write: typing.Optional[str] = None,
                stdout: typing.Union[typing.BinaryIO, int] = subprocess.PIPE,
                stderr: typing.Union[typing.BinaryIO, int] = subprocess.PIPE,
@@ -1389,13 +1398,13 @@ class BuildDirLock:
             msvcrt.locking(self.lockfile.fileno(), msvcrt.LK_UNLCK, 1)
         self.lockfile.close()
 
-def relpath(path, start):
+def relpath(path: str, start: str) -> str:
     # On Windows a relative path can't be evaluated for paths on two different
     # drives (i.e. c:\foo and f:\bar).  The only thing left to do is to use the
     # original absolute path.
     try:
         return os.path.relpath(path, start)
-    except ValueError:
+    except (TypeError, ValueError):
         return path
 
 
@@ -1409,8 +1418,14 @@ class LibType(Enum):
     PREFER_STATIC = 3
 
 
-class ProgressBarFallback:
-    '''Fallback progress bar implementation when tqdm is not found'''
+class ProgressBarFallback:  # lgtm [py/iter-returns-non-self]
+    '''
+    Fallback progress bar implementation when tqdm is not found
+
+    Since this class is not an actual iterator, but only provides a minimal
+    fallback, it is safe to ignore the 'Iterator does not return self from
+    __iter__ method' warning.
+    '''
     def __init__(self, iterable=None, total=None, bar_type=None, desc=None):
         if iterable is not None:
             self.iterable = iter(iterable)
