@@ -18,8 +18,19 @@ import sys
 import time
 import platform
 from contextlib import contextmanager
-import typing
-from typing import Any, Generator, List, Optional, Sequence, TextIO, Union
+from typing import (
+    Any,
+    Generator,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    TextIO,
+    Tuple,
+    Union,
+    cast,
+)
+from pathlib import Path
 
 """This is (mostly) a standalone module used to write logging
 information about Meson runs. Some output goes to screen,
@@ -55,6 +66,8 @@ log_timestamp_start = None   # type: Optional[float]
 log_fatal_warnings = False   # type: bool
 log_disable_stdout = False   # type: bool
 log_errors_only = False      # type: bool
+_in_ci = 'CI' in os.environ  # type: bool
+_logged_once = set()         # type: Set[Tuple[str, ...]]
 
 def disable() -> None:
     global log_disable_stdout
@@ -126,6 +139,21 @@ def blue(text: str) -> AnsiDecorator:
 def cyan(text: str) -> AnsiDecorator:
     return AnsiDecorator(text, "\033[1;36m")
 
+def normal_red(text: str) -> AnsiDecorator:
+    return AnsiDecorator(text, "\033[31m")
+
+def normal_green(text: str) -> AnsiDecorator:
+    return AnsiDecorator(text, "\033[32m")
+
+def normal_yellow(text: str) -> AnsiDecorator:
+    return AnsiDecorator(text, "\033[33m")
+
+def normal_blue(text: str) -> AnsiDecorator:
+    return AnsiDecorator(text, "\033[34m")
+
+def normal_cyan(text: str) -> AnsiDecorator:
+    return AnsiDecorator(text, "\033[36m")
+
 # This really should be AnsiDecorator or anything that implements
 # __str__(), but that requires protocols from typing_extensions
 def process_markup(args: Sequence[Union[AnsiDecorator, str]], keep: bool) -> List[str]:
@@ -144,7 +172,6 @@ def process_markup(args: Sequence[Union[AnsiDecorator, str]], keep: bool) -> Lis
     return arr
 
 def force_print(*args: str, **kwargs: Any) -> None:
-    global log_disable_stdout
     if log_disable_stdout:
         return
     iostr = io.StringIO()
@@ -163,16 +190,24 @@ def force_print(*args: str, **kwargs: Any) -> None:
         cleaned = raw.encode('ascii', 'replace').decode('ascii')
         print(cleaned, end='')
 
-# We really want a heterogenous dict for this, but that's in typing_extensions
+# We really want a heterogeneous dict for this, but that's in typing_extensions
 def debug(*args: Union[str, AnsiDecorator], **kwargs: Any) -> None:
     arr = process_markup(args, False)
     if log_file is not None:
         print(*arr, file=log_file, **kwargs)
         log_file.flush()
 
+def _debug_log_cmd(cmd: str, args: List[str]) -> None:
+    if not _in_ci:
+        return
+    args = ['"{}"'.format(x) for x in args]  # Quote all args, just in case
+    debug('!meson_ci!/{} {}'.format(cmd, ' '.join(args)))
+
+def cmd_ci_include(file: str) -> None:
+    _debug_log_cmd('ci_include', [file])
+
 def log(*args: Union[str, AnsiDecorator], is_error: bool = False,
         **kwargs: Any) -> None:
-    global log_errors_only
     arr = process_markup(args, False)
     if log_file is not None:
         print(*arr, file=log_file, **kwargs)
@@ -182,7 +217,21 @@ def log(*args: Union[str, AnsiDecorator], is_error: bool = False,
     if not log_errors_only or is_error:
         force_print(*arr, **kwargs)
 
-def _log_error(severity: str, *rargs: Union[str, AnsiDecorator], **kwargs: Any) -> None:
+def log_once(*args: Union[str, AnsiDecorator], is_error: bool = False,
+             **kwargs: Any) -> None:
+    """Log variant that only prints a given message one time per meson invocation.
+
+    This considers nasi decorated values by the values they wrap without
+    regard for the AnsiDecorator itself.
+    """
+    t = tuple(a.text if isinstance(a, AnsiDecorator) else a for a in args)
+    if t in _logged_once:
+        return
+    _logged_once.add(t)
+    log(*args, is_error=is_error, **kwargs)
+
+def _log_error(severity: str, *rargs: Union[str, AnsiDecorator],
+               once: bool = False, **kwargs: Any) -> None:
     from .mesonlib import get_error_location_string
     from .environment import build_filename
     from .mesonlib import MesonException
@@ -206,23 +255,40 @@ def _log_error(severity: str, *rargs: Union[str, AnsiDecorator], **kwargs: Any)
         location_str = get_error_location_string(location_file, location.lineno)
         # Unions are frankly awful, and we have to cast here to get mypy
         # to understand that the list concatenation is safe
-        location_list = typing.cast(List[Union[str, AnsiDecorator]], [location_str])
+        location_list = cast(List[Union[str, AnsiDecorator]], [location_str])
         args = location_list + args
 
-    log(*args, **kwargs)
+    if once:
+        log_once(*args, **kwargs)
+    else:
+        log(*args, **kwargs)
 
-    global log_fatal_warnings
     if log_fatal_warnings:
         raise MesonException("Fatal warnings enabled, aborting")
 
-def error(*args: Union[str, AnsiDecorator], **kwargs: Any) -> None:
-    return _log_error('error', *args, **kwargs, is_error=True)
+def error(*args: Union[str, AnsiDecorator], once: bool = False, **kwargs: Any) -> None:
+    return _log_error('error', *args, **kwargs, is_error=True, once=once)
 
-def warning(*args: Union[str, AnsiDecorator], **kwargs: Any) -> None:
-    return _log_error('warning', *args, **kwargs, is_error=True)
+def warning(*args: Union[str, AnsiDecorator], once: bool = False, **kwargs: Any) -> None:
+    return _log_error('warning', *args, **kwargs, is_error=True, once=once)
 
-def deprecation(*args: Union[str, AnsiDecorator], **kwargs: Any) -> None:
-    return _log_error('deprecation', *args, **kwargs, is_error=True)
+def deprecation(*args: Union[str, AnsiDecorator], once: bool = False, **kwargs: Any) -> None:
+    return _log_error('deprecation', *args, **kwargs, is_error=True, once=once)
+
+def get_relative_path(target: Path, current: Path) -> Path:
+    """Get the path to target from current"""
+    # Go up "current" until we find a common ancestor to target
+    acc = ['.']
+    for part in [current, *current.parents]:
+        try:
+            path = target.relative_to(part)
+            return Path(*acc, path)
+        except ValueError:
+            pass
+        acc += ['..']
+
+    # we failed, should not get here
+    return target
 
 def exception(e: Exception, prefix: Optional[AnsiDecorator] = None) -> None:
     if prefix is None:
@@ -232,7 +298,8 @@ def exception(e: Exception, prefix: Optional[AnsiDecorator] = None) -> None:
     if hasattr(e, 'file') and hasattr(e, 'lineno') and hasattr(e, 'colno'):
         # Mypy can't figure this out, and it's pretty easy to vidual inspect
         # that this is correct, so we'll just ignore it.
-        args.append('%s:%d:%d:' % (e.file, e.lineno, e.colno))  # type: ignore
+        path = get_relative_path(Path(e.file), Path(os.getcwd()))
+        args.append('%s:%d:%d:' % (path, e.lineno, e.colno))  # type: ignore
     if prefix:
         args.append(prefix)
     args.append(str(e))
