@@ -14,16 +14,29 @@
 
 from .. import mlog
 import contextlib
-import urllib.request, os, hashlib, shutil, tempfile, stat
+import urllib.request
+import urllib.error
+import urllib.parse
+import os
+import hashlib
+import shutil
+import tempfile
+import stat
 import subprocess
 import sys
 import configparser
+import typing
 
 from . import WrapMode
 from ..mesonlib import ProgressBar, MesonException
 
+if typing.TYPE_CHECKING:
+    import http.client
+
 try:
-    import ssl
+    # Importing is just done to check if SSL exists, so all warnings
+    # regarding 'imported but unused' can be safely ignored
+    import ssl  # noqa
     has_ssl = True
     API_ROOT = 'https://wrapdb.mesonbuild.com/v1/'
 except ImportError:
@@ -32,44 +45,50 @@ except ImportError:
 
 req_timeout = 600.0
 ssl_warning_printed = False
+whitelist_subdomain = 'wrapdb.mesonbuild.com'
 
-def build_ssl_context():
-    ctx = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-    ctx.options |= ssl.OP_NO_SSLv2
-    ctx.options |= ssl.OP_NO_SSLv3
-    ctx.verify_mode = ssl.CERT_REQUIRED
-    ctx.load_default_certs()
-    return ctx
 
-def quiet_git(cmd, workingdir):
-    try:
-        pc = subprocess.Popen(['git', '-C', workingdir] + cmd, stdin=subprocess.DEVNULL,
-                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    except FileNotFoundError as e:
-        return False, str(e)
-    out, err = pc.communicate()
+def quiet_git(cmd: typing.List[str], workingdir: str) -> typing.Tuple[bool, str]:
+    git = shutil.which('git')
+    if not git:
+        return False, 'Git program not found.'
+    pc = subprocess.run([git, '-C', workingdir] + cmd, universal_newlines=True,
+                        stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     if pc.returncode != 0:
-        return False, err
-    return True, out
+        return False, pc.stderr
+    return True, pc.stdout
 
-def open_wrapdburl(urlstring):
+def whitelist_wrapdb(urlstr: str) -> urllib.parse.ParseResult:
+    """ raises WrapException if not whitelisted subdomain """
+    url = urllib.parse.urlparse(urlstr)
+    if not url.hostname:
+        raise WrapException('{} is not a valid URL'.format(urlstr))
+    if not url.hostname.endswith(whitelist_subdomain):
+        raise WrapException('{} is not a whitelisted WrapDB URL'.format(urlstr))
+    if has_ssl and not url.scheme == 'https':
+        raise WrapException('WrapDB did not have expected SSL https url, instead got {}'.format(urlstr))
+    return url
+
+def open_wrapdburl(urlstring: str) -> 'http.client.HTTPResponse':
     global ssl_warning_printed
+
+    url = whitelist_wrapdb(urlstring)
     if has_ssl:
         try:
-            return urllib.request.urlopen(urlstring, timeout=req_timeout)# , context=build_ssl_context())
-        except urllib.error.URLError:
-            if not ssl_warning_printed:
-                print('SSL connection failed. Falling back to unencrypted connections.')
-                ssl_warning_printed = True
+            return urllib.request.urlopen(urllib.parse.urlunparse(url), timeout=req_timeout)
+        except urllib.error.URLError as excp:
+            raise WrapException('WrapDB connection failed to {} with error {}'.format(urlstring, excp))
+
+    # following code is only for those without Python SSL
+    nossl_url = url._replace(scheme='http')
     if not ssl_warning_printed:
-        print('Warning: SSL not available, traffic not authenticated.',
-              file=sys.stderr)
+        mlog.warning('SSL module not available in {}: WrapDB traffic not authenticated.'.format(sys.executable))
         ssl_warning_printed = True
-    # Trying to open SSL connection to wrapdb fails because the
-    # certificate is not known.
-    if urlstring.startswith('https'):
-        urlstring = 'http' + urlstring[5:]
-    return urllib.request.urlopen(urlstring, timeout=req_timeout)
+    try:
+        return urllib.request.urlopen(urllib.parse.urlunparse(nossl_url), timeout=req_timeout)
+    except urllib.error.URLError as excp:
+        raise WrapException('WrapDB connection failed to {} with error {}'.format(urlstring, excp))
+
 
 class WrapException(MesonException):
     pass
@@ -78,7 +97,7 @@ class WrapNotFoundException(WrapException):
     pass
 
 class PackageDefinition:
-    def __init__(self, fname):
+    def __init__(self, fname: str):
         self.filename = fname
         self.basename = os.path.basename(fname)
         self.name = self.basename[:-5]
@@ -96,32 +115,42 @@ class PackageDefinition:
         self.type = self.wrap_section[5:]
         self.values = dict(self.config[self.wrap_section])
 
-    def get(self, key):
+    def get(self, key: str) -> str:
         try:
             return self.values[key]
         except KeyError:
             m = 'Missing key {!r} in {}'
             raise WrapException(m.format(key, self.basename))
 
-    def has_patch(self):
+    def has_patch(self) -> bool:
         return 'patch_url' in self.values
 
+def load_wrap(subdir_root: str, packagename: str) -> PackageDefinition:
+    fname = os.path.join(subdir_root, packagename + '.wrap')
+    if os.path.isfile(fname):
+        return PackageDefinition(fname)
+    return None
+
+def get_directory(subdir_root: str, packagename: str):
+    directory = packagename
+    # We always have to load the wrap file, if it exists, because it could
+    # override the default directory name.
+    wrap = load_wrap(subdir_root, packagename)
+    if wrap and 'directory' in wrap.values:
+        directory = wrap.get('directory')
+        if os.path.dirname(directory):
+            raise WrapException('Directory key must be a name and not a path')
+    return wrap, directory
+
 class Resolver:
-    def __init__(self, subdir_root, wrap_mode=WrapMode.default):
+    def __init__(self, subdir_root: str, wrap_mode=WrapMode.default):
         self.wrap_mode = wrap_mode
         self.subdir_root = subdir_root
         self.cachedir = os.path.join(self.subdir_root, 'packagecache')
 
-    def resolve(self, packagename: str, method: str):
+    def resolve(self, packagename: str, method: str) -> str:
         self.packagename = packagename
-        self.directory = packagename
-        # We always have to load the wrap file, if it exists, because it could
-        # override the default directory name.
-        self.wrap = self.load_wrap()
-        if self.wrap and 'directory' in self.wrap.values:
-            self.directory = self.wrap.get('directory')
-            if os.path.dirname(self.directory):
-                raise WrapException('Directory key must be a name and not a path')
+        self.wrap, self.directory = get_directory(self.subdir_root, self.packagename)
         self.dirname = os.path.join(self.subdir_root, self.directory)
         meson_file = os.path.join(self.dirname, 'meson.build')
         cmake_file = os.path.join(self.dirname, 'CMakeLists.txt')
@@ -168,20 +197,17 @@ class Resolver:
 
         return self.directory
 
-    def load_wrap(self):
-        fname = os.path.join(self.subdir_root, self.packagename + '.wrap')
-        if os.path.isfile(fname):
-            return PackageDefinition(fname)
-        return None
-
-    def check_can_download(self):
+    def check_can_download(self) -> None:
         # Don't download subproject data based on wrap file if requested.
         # Git submodules are ok (see above)!
         if self.wrap_mode is WrapMode.nodownload:
             m = 'Automatic wrap-based subproject downloading is disabled'
             raise WrapException(m)
 
-    def resolve_git_submodule(self):
+    def resolve_git_submodule(self) -> bool:
+        git = shutil.which('git')
+        if not git:
+            raise WrapException('Git program not found.')
         # Are we in a git repository?
         ret, out = quiet_git(['rev-parse'], self.subdir_root)
         if not ret:
@@ -191,29 +217,30 @@ class Resolver:
         if not ret:
             return False
         # Submodule has not been added, add it
-        if out.startswith(b'+'):
+        if out.startswith('+'):
             mlog.warning('git submodule might be out of date')
             return True
-        elif out.startswith(b'U'):
+        elif out.startswith('U'):
             raise WrapException('git submodule has merge conflicts')
         # Submodule exists, but is deinitialized or wasn't initialized
-        elif out.startswith(b'-'):
-            if subprocess.call(['git', '-C', self.subdir_root, 'submodule', 'update', '--init', self.dirname]) == 0:
+        elif out.startswith('-'):
+            if subprocess.run([git, '-C', self.subdir_root,
+                              'submodule', 'update', '--init', self.dirname]).returncode == 0:
                 return True
             raise WrapException('git submodule failed to init')
         # Submodule looks fine, but maybe it wasn't populated properly. Do a checkout.
-        elif out.startswith(b' '):
-            subprocess.call(['git', 'checkout', '.'], cwd=self.dirname)
+        elif out.startswith(' '):
+            subprocess.run([git, 'checkout', '.'], cwd=self.dirname)
             # Even if checkout failed, try building it anyway and let the user
             # handle any problems manually.
             return True
-        elif out == b'':
+        elif out == '':
             # It is not a submodule, just a folder that exists in the main repository.
             return False
         m = 'Unknown git submodule output: {!r}'
         raise WrapException(m.format(out))
 
-    def get_file(self):
+    def get_file(self) -> None:
         path = self.get_file_internal('source')
         extract_dir = self.subdir_root
         # Some upstreams ship packages that do not have a leading directory.
@@ -225,7 +252,10 @@ class Resolver:
         if self.wrap.has_patch():
             self.apply_patch()
 
-    def get_git(self):
+    def get_git(self) -> None:
+        git = shutil.which('git')
+        if not git:
+            raise WrapException('Git program not found.')
         revno = self.wrap.get('revision')
         is_shallow = False
         depth_option = []    # type: typing.List[str]
@@ -236,72 +266,86 @@ class Resolver:
         if is_shallow and self.is_git_full_commit_id(revno):
             # git doesn't support directly cloning shallowly for commits,
             # so we follow https://stackoverflow.com/a/43136160
-            subprocess.check_call(['git', 'init', self.directory], cwd=self.subdir_root)
-            subprocess.check_call(['git', 'remote', 'add', 'origin', self.wrap.get('url')],
+            subprocess.check_call([git, 'init', self.directory], cwd=self.subdir_root)
+            subprocess.check_call([git, 'remote', 'add', 'origin', self.wrap.get('url')],
                                   cwd=self.dirname)
             revno = self.wrap.get('revision')
-            subprocess.check_call(['git', 'fetch', *depth_option, 'origin', revno],
+            subprocess.check_call([git, 'fetch', *depth_option, 'origin', revno],
                                   cwd=self.dirname)
-            subprocess.check_call(['git', 'checkout', revno], cwd=self.dirname)
+            subprocess.check_call([git, 'checkout', revno], cwd=self.dirname)
             if self.wrap.values.get('clone-recursive', '').lower() == 'true':
-                subprocess.check_call(['git', 'submodule', 'update',
+                subprocess.check_call([git, 'submodule', 'update',
                                        '--init', '--checkout', '--recursive', *depth_option],
                                       cwd=self.dirname)
             push_url = self.wrap.values.get('push-url')
             if push_url:
-                subprocess.check_call(['git', 'remote', 'set-url',
+                subprocess.check_call([git, 'remote', 'set-url',
                                        '--push', 'origin', push_url],
                                       cwd=self.dirname)
         else:
             if not is_shallow:
-                subprocess.check_call(['git', 'clone', self.wrap.get('url'),
+                subprocess.check_call([git, 'clone', self.wrap.get('url'),
                                        self.directory], cwd=self.subdir_root)
                 if revno.lower() != 'head':
-                    if subprocess.call(['git', 'checkout', revno], cwd=self.dirname) != 0:
-                        subprocess.check_call(['git', 'fetch', self.wrap.get('url'), revno], cwd=self.dirname)
-                        subprocess.check_call(['git', 'checkout', revno], cwd=self.dirname)
+                    if subprocess.run([git, 'checkout', revno], cwd=self.dirname).returncode != 0:
+                        subprocess.check_call([git, 'fetch', self.wrap.get('url'), revno], cwd=self.dirname)
+                        subprocess.check_call([git, 'checkout', revno], cwd=self.dirname)
             else:
-                subprocess.check_call(['git', 'clone', *depth_option,
+                subprocess.check_call([git, 'clone', *depth_option,
                                        '--branch', revno,
                                        self.wrap.get('url'),
                                        self.directory], cwd=self.subdir_root)
             if self.wrap.values.get('clone-recursive', '').lower() == 'true':
-                subprocess.check_call(['git', 'submodule', 'update',
+                subprocess.check_call([git, 'submodule', 'update',
                                        '--init', '--checkout', '--recursive', *depth_option],
                                       cwd=self.dirname)
             push_url = self.wrap.values.get('push-url')
             if push_url:
-                subprocess.check_call(['git', 'remote', 'set-url',
+                subprocess.check_call([git, 'remote', 'set-url',
                                        '--push', 'origin', push_url],
                                       cwd=self.dirname)
 
-    def is_git_full_commit_id(self, revno):
+    def is_git_full_commit_id(self, revno: str) -> bool:
         result = False
         if len(revno) in (40, 64): # 40 for sha1, 64 for upcoming sha256
             result = all((ch in '0123456789AaBbCcDdEeFf' for ch in revno))
         return result
 
-    def get_hg(self):
+    def get_hg(self) -> None:
         revno = self.wrap.get('revision')
-        subprocess.check_call(['hg', 'clone', self.wrap.get('url'),
+        hg = shutil.which('hg')
+        if not hg:
+            raise WrapException('Mercurial program not found.')
+        subprocess.check_call([hg, 'clone', self.wrap.get('url'),
                                self.directory], cwd=self.subdir_root)
         if revno.lower() != 'tip':
-            subprocess.check_call(['hg', 'checkout', revno],
+            subprocess.check_call([hg, 'checkout', revno],
                                   cwd=self.dirname)
 
-    def get_svn(self):
+    def get_svn(self) -> None:
         revno = self.wrap.get('revision')
-        subprocess.check_call(['svn', 'checkout', '-r', revno, self.wrap.get('url'),
+        svn = shutil.which('svn')
+        if not svn:
+            raise WrapException('SVN program not found.')
+        subprocess.check_call([svn, 'checkout', '-r', revno, self.wrap.get('url'),
                                self.directory], cwd=self.subdir_root)
 
-    def get_data(self, url):
+    def get_data(self, urlstring: str) -> typing.Tuple[str, str]:
         blocksize = 10 * 1024
         h = hashlib.sha256()
         tmpfile = tempfile.NamedTemporaryFile(mode='wb', dir=self.cachedir, delete=False)
-        if url.startswith('https://wrapdb.mesonbuild.com'):
-            resp = open_wrapdburl(url)
+        url = urllib.parse.urlparse(urlstring)
+        if not url.hostname:
+            raise WrapException('{} is not a valid URL'.format(urlstring))
+        if url.hostname.endswith(whitelist_subdomain):
+            resp = open_wrapdburl(urlstring)
+        elif whitelist_subdomain in urlstring:
+            raise WrapException('{} may be a WrapDB-impersonating URL'.format(urlstring))
         else:
-            resp = urllib.request.urlopen(url, timeout=req_timeout)
+            try:
+                resp = urllib.request.urlopen(urlstring, timeout=req_timeout)
+            except urllib.error.URLError:
+                raise WrapException('could not get {} is the internet available?'.format(urlstring))
         with contextlib.closing(resp) as resp:
             try:
                 dlsize = int(resp.info()['Content-Length'])
@@ -331,16 +375,16 @@ class Resolver:
             hashvalue = h.hexdigest()
         return hashvalue, tmpfile.name
 
-    def check_hash(self, what, path):
+    def check_hash(self, what: str, path: str) -> None:
         expected = self.wrap.get(what + '_hash')
         h = hashlib.sha256()
         with open(path, 'rb') as f:
             h.update(f.read())
         dhash = h.hexdigest()
         if dhash != expected:
-            raise WrapException('Incorrect hash for %s:\n %s expected\n %s actual.' % (what, expected, dhash))
+            raise WrapException('Incorrect hash for {}:\n {} expected\n {} actual.'.format(what, expected, dhash))
 
-    def download(self, what, ofname):
+    def download(self, what: str, ofname: str) -> None:
         self.check_can_download()
         srcurl = self.wrap.get(what + '_url')
         mlog.log('Downloading', mlog.bold(self.packagename), what, 'from', mlog.bold(srcurl))
@@ -348,10 +392,10 @@ class Resolver:
         expected = self.wrap.get(what + '_hash')
         if dhash != expected:
             os.remove(tmpfile)
-            raise WrapException('Incorrect hash for %s:\n %s expected\n %s actual.' % (what, expected, dhash))
+            raise WrapException('Incorrect hash for {}:\n {} expected\n {} actual.'.format(what, expected, dhash))
         os.rename(tmpfile, ofname)
 
-    def get_file_internal(self, what):
+    def get_file_internal(self, what: str) -> str:
         filename = self.wrap.get(what + '_filename')
         cache_path = os.path.join(self.cachedir, filename)
 
@@ -365,7 +409,7 @@ class Resolver:
         self.download(what, cache_path)
         return cache_path
 
-    def apply_patch(self):
+    def apply_patch(self) -> None:
         path = self.get_file_internal('patch')
         try:
             shutil.unpack_archive(path, self.subdir_root)
@@ -374,7 +418,7 @@ class Resolver:
                 shutil.unpack_archive(path, workdir)
                 self.copy_tree(workdir, self.subdir_root)
 
-    def copy_tree(self, root_src_dir, root_dst_dir):
+    def copy_tree(self, root_src_dir: str, root_dst_dir: str) -> None:
         """
         Copy directory tree. Overwrites also read only files.
         """
